@@ -15,9 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: pdm-dialog.c,v 1.66 2006/12/15 14:01:58 chpe Exp $
+ *  $Id$
  */
 
 #include "config.h"
@@ -32,21 +32,14 @@
 #include "ephy-string.h"
 #include "ephy-stock-icons.h"
 #include "ephy-debug.h"
+#include "ephy-time-helpers.h"
+#include "ephy-embed-single.h"
+#include "ephy-favicon-cache.h"
+#include "ephy-history.h"
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtktogglebutton.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <time.h>
 #include <string.h>
 
 typedef struct PdmActionInfo PdmActionInfo;
@@ -119,6 +112,11 @@ enum
 	PROP_PASSWORDS_SHOW
 };
 
+enum
+{
+	PDM_DIALOG_RESPONSE_CLEAR = 1
+};
+
 static const
 EphyDialogProperty properties [] =
 {
@@ -139,43 +137,17 @@ static void pdm_dialog_class_init	(PdmDialogClass *klass);
 static void pdm_dialog_init		(PdmDialog *dialog);
 static void pdm_dialog_finalize		(GObject *object);
 
-static GObjectClass *parent_class = NULL;
+static void passwords_changed_cb 		(EphyPasswordManager *manager,
+						 PdmDialog *dialog);
 
-GType 
-pdm_dialog_get_type (void)
-{
-	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (PdmDialogClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) pdm_dialog_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (PdmDialog),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) pdm_dialog_init
-		};
-
-		type = g_type_register_static (EPHY_TYPE_DIALOG,
-					       "PdmDialog",
-					       &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (PdmDialog, pdm_dialog, EPHY_TYPE_DIALOG)
 
 static void
 pdm_dialog_class_init (PdmDialogClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = pdm_dialog_finalize;
 
 	g_type_class_add_private (object_class, sizeof(PdmDialogPrivate));
@@ -204,6 +176,256 @@ pdm_dialog_show_help (PdmDialog *pd)
 	ephy_gui_help (GTK_WINDOW (window), "epiphany", help_preferences[id]);
 }
 
+typedef struct
+{
+	EphyDialog *dialog;
+	GtkWidget *checkbutton_history;
+	GtkWidget *checkbutton_cookies;
+	GtkWidget *checkbutton_passwords;
+	GtkWidget *checkbutton_cache;
+	guint num_checked;
+} PdmClearAllDialogButtons;
+
+static void
+clear_all_dialog_release_cb (PdmClearAllDialogButtons *data)
+{
+	g_slice_free (PdmClearAllDialogButtons, data);
+}
+
+static void
+clear_all_dialog_response_cb (GtkDialog *dialog,
+			      int response,
+			      PdmClearAllDialogButtons *checkbuttons)
+{
+	if (response == GTK_RESPONSE_HELP)
+	{
+		/* Show help and return early */
+
+		ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "clearing-personal-data");
+		return;
+	}
+
+	if (response == GTK_RESPONSE_OK)
+	{
+		if (gtk_toggle_button_get_active 
+		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_history)))
+		{
+			EphyEmbedShell *shell;
+			EphyHistory *history;
+
+			shell = ephy_embed_shell_get_default ();
+			history = EPHY_HISTORY (ephy_embed_shell_get_global_history (shell));
+			ephy_history_clear (history);
+		}
+		if (gtk_toggle_button_get_active
+		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cookies)))
+		{
+			EphyCookieManager *manager;
+
+			manager = EPHY_COOKIE_MANAGER (ephy_embed_shell_get_embed_single
+						       (EPHY_EMBED_SHELL (ephy_shell)));
+
+			ephy_cookie_manager_clear (manager);
+		}
+		if (gtk_toggle_button_get_active
+		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_passwords)))
+		{
+			EphyPasswordManager *manager;
+
+			manager = EPHY_PASSWORD_MANAGER (ephy_embed_shell_get_embed_single
+							 (EPHY_EMBED_SHELL (ephy_shell)));
+
+			/* we don't remove the password from the liststore in the callback
+			 * like we do for cookies, since the callback doesn't carry that
+			 * information, and we'd have to reload the whole list, losing the
+			 * selection in the process.
+			 */
+
+			if (EPHY_IS_PDM_DIALOG (checkbuttons->dialog))
+			{
+				g_signal_handlers_block_by_func
+					(manager, G_CALLBACK (passwords_changed_cb), 
+					 EPHY_PDM_DIALOG (checkbuttons->dialog));
+			}
+
+			/* Flush the list */
+			ephy_password_manager_remove_all_passwords (manager);
+
+			if (EPHY_IS_PDM_DIALOG (checkbuttons->dialog))
+			{
+				g_signal_handlers_unblock_by_func
+					(manager, G_CALLBACK (passwords_changed_cb),
+					 EPHY_PDM_DIALOG (checkbuttons->dialog));
+
+				/* And now refresh explicitly */
+				passwords_changed_cb (manager, EPHY_PDM_DIALOG (checkbuttons->dialog));
+			}
+		}
+		if (gtk_toggle_button_get_active
+		    	(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cache)))
+		{
+			EphyEmbedShell *shell;
+			EphyEmbedSingle *single;
+			EphyFaviconCache *cache;
+
+			shell = ephy_embed_shell_get_default ();
+
+			single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (shell));
+
+			ephy_embed_single_clear_cache (single);
+
+			cache = EPHY_FAVICON_CACHE (ephy_embed_shell_get_favicon_cache (shell));
+			ephy_favicon_cache_clear (cache);
+		}
+	}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+clear_all_dialog_checkbutton_toggled_cb (GtkToggleButton *toggle,
+					 PdmClearAllDialogButtons *data)
+{
+	GtkWidget *dialog;
+	dialog = gtk_widget_get_toplevel (GTK_WIDGET (toggle));
+	
+	if (gtk_toggle_button_get_active (toggle) == TRUE)
+	{
+		data->num_checked++;
+	}
+	else
+	{
+		data->num_checked--;
+	}
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK,
+					   data->num_checked != 0);
+}
+void
+pdm_dialog_show_clear_all_dialog (EphyDialog *edialog,
+				  GtkWidget *parent,
+				  PdmClearAllDialogFlags flags)
+{
+	GtkWidget *dialog, *vbox;
+	GtkWidget *check, *label;
+	PdmClearAllDialogButtons *checkbuttons;
+	GtkWidget *button, *icon;
+
+	dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (parent),
+						     GTK_DIALOG_DESTROY_WITH_PARENT |
+						     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_QUESTION,
+						     GTK_BUTTONS_NONE,
+						     _("<b>Select the personal data "
+						       "you want to clear</b>"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("You are about to clear personal data "
+						    "that is stored about the web pages "
+						    "you have visited. Before proceeding, "
+						    "check the types of information that you "
+						    "want to remove:"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Clear All Personal Data"));
+
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+				GTK_STOCK_HELP,
+				GTK_RESPONSE_HELP,
+				GTK_STOCK_CANCEL,
+				GTK_RESPONSE_CANCEL,
+				NULL);
+
+	/* Clear button */
+	button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+					_("Cl_ear"),
+					GTK_RESPONSE_OK);
+	icon = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image (GTK_BUTTON (button), icon);
+	gtk_widget_show (button);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+					 GTK_RESPONSE_CANCEL);
+	gtk_label_set_selectable (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
+				  FALSE);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (GTK_MESSAGE_DIALOG (dialog)->label->parent),
+			    vbox, FALSE, FALSE, 0);
+
+	checkbuttons = g_slice_new0 (PdmClearAllDialogButtons);
+	checkbuttons->dialog = edialog;
+	checkbuttons->num_checked = 0;
+
+	/* Cookies */
+	check = gtk_check_button_new_with_mnemonic (_("C_ookies"));
+	checkbuttons->checkbutton_cookies = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_COOKIES)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* Passwords */
+	check = gtk_check_button_new_with_mnemonic (_("Saved _passwords"));
+	checkbuttons->checkbutton_passwords = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_PASSWORDS)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* History */
+	check = gtk_check_button_new_with_mnemonic (_("Hi_story"));
+	checkbuttons->checkbutton_history = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_HISTORY)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+	
+	/* Cache */
+	check = gtk_check_button_new_with_mnemonic (_("_Temporary files"));
+	checkbuttons->checkbutton_cache = check;
+	gtk_box_pack_start (GTK_BOX (vbox), check,
+			    FALSE, FALSE, 0);
+	g_signal_connect (check, "toggled",
+			  G_CALLBACK (clear_all_dialog_checkbutton_toggled_cb), checkbuttons);
+	if (flags & CLEAR_ALL_CACHE)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+	}
+
+	/* Show vbox and all checkbuttons */
+	gtk_widget_show_all (vbox);
+
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label),
+			      _("<small><i><b>Note:</b> You cannot undo this action. "
+			        "The data you are choosing to clear "
+				"will be deleted forever.</i></small>"));
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+			    label, FALSE, FALSE, 0);
+	/* Need to do this or the label will wrap too early */
+	gtk_widget_set_size_request (label, 330, -1);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (label),
+				0, 0);
+	gtk_misc_set_padding (GTK_MISC (label),
+			      6, 0);
+	gtk_widget_show (label);
+
+	gtk_window_present (GTK_WINDOW (dialog));
+	g_signal_connect_data (dialog, "response",
+			       G_CALLBACK (clear_all_dialog_response_cb),
+			       checkbuttons, (GClosureNotify) clear_all_dialog_release_cb,
+			       (GConnectFlags) 0);
+}
+
 static void
 action_treeview_selection_changed_cb (GtkTreeSelection *selection,
 				      PdmActionInfo *action)
@@ -216,6 +438,7 @@ action_treeview_selection_changed_cb (GtkTreeSelection *selection,
 
 	widget = ephy_dialog_get_control (d, properties[action->remove_id].id);
 	gtk_widget_set_sensitive (widget, has_selection);
+
 }
 
 static void
@@ -333,6 +556,9 @@ pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 				     PdmActionInfo *action)
 {
 	pdm_cmd_delete_selection (action);
+
+	/* Restore the focus to the button */
+	gtk_widget_grab_focus (button);
 }
 
 static void
@@ -356,7 +582,6 @@ setup_action (PdmActionInfo *action)
 			  "key_press_event",
 			  G_CALLBACK (pdm_key_pressed_cb),
 			  action);
-
 }
 
 /* "Cookies" tab */
@@ -456,10 +681,7 @@ show_cookies_properties (PdmDialog *dialog,
 	else
 	{
 		struct tm t;
-		char s[128];
-		const char *fmt_hack = "%c";
-		strftime (s, sizeof(s), fmt_hack, localtime_r (&info->expires, &t));
-		str = g_locale_to_utf8 (s, -1, NULL, NULL, NULL);
+		str = eel_strdup_strftime ("%c", localtime_r (&info->expires, &t));
 	}
 	label = gtk_label_new (str);
 	g_free (str);
@@ -579,6 +801,7 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	info->model = GTK_TREE_MODEL (liststore);
 	g_object_unref (liststore);
 
+        cookies_treeview_selection_changed_cb (selection, dialog);
 	g_signal_connect (selection, "changed",
 			  G_CALLBACK(cookies_treeview_selection_changed_cb),
 			  dialog);
@@ -1180,6 +1403,29 @@ pdm_dialog_response_cb (GtkDialog *widget,
 		pdm_dialog_show_help (dialog);
 		return;
 	}
+	if (response == PDM_DIALOG_RESPONSE_CLEAR)
+	{
+		int page;
+		GtkWidget *parent;
+		
+		parent = ephy_dialog_get_control (EPHY_DIALOG (dialog),
+						  properties[PROP_WINDOW].id);
+
+		page = gtk_notebook_get_current_page (GTK_NOTEBOOK (dialog->priv->notebook));
+		if (page == 0)
+		{
+			/* Cookies */
+			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog), 
+							  parent, CLEAR_ALL_COOKIES);
+		}
+		if (page == 1)
+		{
+			/* Passwords */
+			pdm_dialog_show_clear_all_dialog (EPHY_DIALOG (dialog),
+							  parent, CLEAR_ALL_PASSWORDS);			
+		}
+		return;
+	}
 
 	g_object_unref (dialog);
 }
@@ -1276,7 +1522,7 @@ pdm_dialog_finalize (GObject *object)
 	g_free (dialog->priv->passwords);
 	g_free (dialog->priv->cookies);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (pdm_dialog_parent_class)->finalize (object);
 }
 
 void
